!=========================================================================
! Copyright (C) GemTalk Systems 1986-2020.  All Rights Reserved.
!
! $Id: gcuser.gs 47588 2020-01-07 18:05:41Z lalmarod $
!
! Description - Create the GcUser and SymbolUser accounts
! 
! Repository conversion - this file is used
!
!=======================================================================

! login as SystemUser
set user SystemUser pass swordfish
! comment out following for debugging
iferr 1 exit	

login 
iferr 1 where
iferr 2 stack

! for repository conversion, fixup default GsObjectSecurityPolicy of DataCurator
!  if needed

expectvalue %String
run
| dc seg2 convFromPrevious64bit report lf |
report := String new .
lf := Character lf .
convFromPrevious64bit := false .
seg2 := (Globals at:#SystemRepository) at: 2 .
dc := AllUsers detect: [ :x | x userId asString = 'DataCurator' ] .

"reset the DataCurator password so the sessionmethods.topaz step 
 can login as DataCurator ."
dc password:'swordfish' .
GsFile gciLogServer:'reset DataCurator password' .
report addAll:'reset DataCurator password'; add: lf .

(dc defaultObjectSecurityPolicy == nil or:[dc defaultObjectSecurityPolicy objectSecurityPolicyId == 1]) ifTrue:[
  convFromPrevious64bit := true .
  dc defaultObjectSecurityPolicy: seg2 .
  GsFile gciLogServer:'changed DataCurator defaultObjectSecurityPolicy' .
  report addAll:'changed DataCurator defaultObjectSecurityPolicy to GsObjectSecurityPolicy with objectSecurityPolicyId 2';
 	add: lf .
] ifFalse:[
  GsFile gciLogServer:'no change to DataCurator defaultObjectSecurityPolicy' .
  report addAll:'no change to DataCurator defaultObjectSecurityPolicy'; add: lf .
].
convFromPrevious64bit ifTrue:[
  AllUsers do:[:up | |uId |
    uId := up userId .
    GsFile gciLogServer:'processing ' , uId .
    (UserProfile isSpecialUserId: uId) ifFalse:[
      report addAll:'changed ', uId, ' defaultObjectSecurityPolicy to nil '; add: lf .
      "set for world-write to new objects per previous Gs64 release."
      up defaultObjectSecurityPolicy: nil . 
    ].
  ].
].
^ report
%


expectvalue %String
run
  " create GcUser account"
| gcUser newUser gcUserGlobals seg7 report lf |

gcUser := AllUsers detect: [ :x | x userId asString = 'GcUser' ] ifNone: [nil].
seg7 := (Globals at:#SystemRepository) at: 7 .
report := String new .
lf := Character lf .
gcUser ~~ nil ifTrue: [ | defSeg |
    "GcUser already exists. Set its password."
    gcUser password: 'swordfish'.
    defSeg := gcUser defaultObjectSecurityPolicy .
    defSeg == seg7 ifFalse:[ 
      gcUser defaultObjectSecurityPolicy: seg7 .
    ].
    seg7 isInvariant ifFalse:[
      seg7 owner: gcUser  .
    ].
    report addAll: 'GcUser already exists' ; add: lf .
] ifFalse: [
    "Create a new GcUser"
    newUser := UserProfile newWithUserId: 'GcUser'
                         password: #swordfish
                         defaultObjectSecurityPolicy: seg7 
                         privileges: #( #GarbageCollection)
                         inGroups: #() .

     seg7 owner: newUser  ;
	ownerAuthorization: #write ;
        worldAuthorization: #read . 

    " AllUsers add: newUser. --- no longer needed "

    " Put GcUser in SystemUser's UserGlobals "
    UserGlobals at: #GcUser put: newUser.

    " Put GcUser in DataCurator's UserGlobals"
    ((AllUsers userWithId: #DataCurator) resolveSymbol: #UserGlobals) value at: 
       #GcUser put: newUser.

    " Put GcUser in its own UserGlobals"
    gcUserGlobals := (newUser resolveSymbol: #UserGlobals) value.
    gcUserGlobals at: #GcUser put: newUser.
    report addAll: 'created GcUser' ; add: lf .
].
seg7 isInvariant ifFalse:[
  seg7 ownerAuthorization: #write ;
    worldAuthorization: #read .

  "Gs64 v2.2, DataCurator needs permission to modify associations in 
   GcUser's userGlobals for running reclaimAll, etc. "
  seg7 group: (AllGroups at:'System') authorization: #write .
] ifTrue:[
   report addAll:'seg7 isInvariant, could not add System group write ';
	add: lf .
].
^ report
%

expectvalue %String
run
  " create SymbolUser account"
| symUser newUser theUg compilerSyms |

"Symbols used by initInitialSelectors in compile.c that don't have other refs.
 This array is an optimization to keeps these symbols from being garbage collected."

compilerSyms := #( #_rubyKindOf: #__inProtectedMode #_isScaledDecimal
		   #_isRubyString #__callerEnvId #__threadRubyEnvId 
                   #signalBreakWith: #_doesNotUnderstand: #_mustBeBoolean 
                   #_gsReturnTos #_changeSegmentFrom:to: 
                   #_deadNscBeingGarbageCollected 
                   #_executeWith:with:with:with: #exclusive 
                   #_canonicalizeSymbolAt:oldSymbol:newSymbol: 
                   #_disableProtectedMode #super #_trapOop 
                   #EmptyString #EmptyArray 
                   #'whileFalse#1__' #'whileTrue#1__'
                   #'untilFalse#1__' #'untilTrue#1__'
                   #'value#0__' #'value#1__'
                  ) .

symUser := AllUsers detect: [ :x | x userId = 'SymbolUser' ] ifNone: [nil].
symUser ifNotNil: [
    "upgrade path, SymbolUser exists "
    symUser password: 'swordfish';
      defaultObjectSecurityPolicy: SystemObjectSecurityPolicy ;
      privileges: #( #GarbageCollection) .
  theUg := (symUser resolveSymbol: #UserGlobals) value.
  theUg at: #CompilerSymbols put: compilerSyms .
    ^ 'SymbolUser already exists, adjusted.'
] ifNil: [
    "Create a new SymbolUser profile " 
    newUser := UserProfile newWithUserId: 'SymbolUser'
                         password: #swordfish
                         defaultObjectSecurityPolicy: SystemObjectSecurityPolicy 
                         privileges: #( #GarbageCollection)
                         inGroups: #( ) .

    " AllUsers add: newUser. --- no longer needed "

    " Put SymbolUser in its own UserGlobals"
    theUg := (newUser resolveSymbol: #UserGlobals) value.
    
    theUg at: #SymbolUser put: newUser.
    theUg at: #PossibleDeadSymbols put: (Object _objectForOop: 248577).
    theUg at: #CompilerSymbols put: compilerSyms .
    ^ 'created SymbolUser'
]
%
commit

logout

set user GcUser pass swordfish
iferr 1 exit	
login 
iferr 1 where
iferr 2 stack

! edited epochGcTimeLimit to fix 34388 
run
UserGlobals at:#defaultValues put:

'
"This String documents the default values for the garbage collector.
 Execute this GemStone Smalltalk source and commit to reset all default values"

"During recovery or restore from logs, the following values are used as defaults:
   deadObjsReclaimedCommitThreshold = 100000.
   objsMovedPerCommitThreshold = 100000.
   reclaimDeadEnabled = true.
   reclaimMinPages = 0.
   sleepTimeBetweenReclaimMs = 0.
   sleepTimeWithCrBacklogMs = 0.
"

" These config values can be changed by committing a new value and the 
  gcGem will start using that value at the next commit/abort that it executes.
  If the system is idle, this should occur within 60 seconds
  Use System getAdminConfig/setAdminConfig:toValue: and 
  getReclaimConfig/setReclaimConfig:toValue: to query and update the 
  configurations at runtime.  This allows the configs to
  be changed during restoreFromLogs when commits are disallowed.
"

"#autoRefreshGcGemConfig is obsolete in 3.3" 
"#enableDebugging is obsolete in 3.3"

" ReclaimGem configuration parameters"

UserGlobals at: #deadObjsReclaimedCommitThreshold put: 20000.
   "Maximum number of dead objects to reclaim in a single transaction.
    Minimum: 32"

UserGlobals at: #deferReclaimCacheDirtyThreshold put: 75.
   "Specifies a threshold percentage of dirty pages in the stones shared
    page cache.  If more than this percentage of pages in the stone
    shared cache are dirty pages, page reclaims will be deferred until 
    the dirty percentage drops below the threshold minus 5%. 
    Setting the value to 100 disables this feature."

UserGlobals at: #maxTransactionDuration put: 300.
   "Controls the approximate maximum length of a reclaim transaction in
    seconds.  If the Reclaim gem has been in transaction longer than
    this duration, it will commit even if the #objsMovedPerCommitThreshold
    condition has not been satisfied.  Minimum value is 10"

UserGlobals at: #objectTableCleanupIdleTime put: 0.
    "Controls whether objectTableCleanup (pushing deltas to leaves) is enabled
     and sets the time in seconds that system is idle (no commits) before the
     reclaim gem attempts to perform the cleanup.
     The default value (zero) is disabled."

UserGlobals at: #objsMovedPerCommitThreshold put: 20000.
   "Controls the approximate number of object table updates to be performed
    per transaction.  Reclaim gems will commit when at least this many
    live objects have been moved to new data pages."

UserGlobals at: #reclaimDeadEnabled  put: true.
  "Controls whether the reclaim will try to clean up dead objects."

UserGlobals at: #reclaimMinFreeSpaceMb put: 0.
   "Minimum repository free space, expressed in megabytes, which must be
    available in order for reclaims to proceed.  Reclaims will be temporarily
    suspended if the repository free space drops below this threshold.
    The default value of 0 specifies a varying reclaimMinFreeSpaceMb that 
    is computed as the current size of the repository divided by 1000, with
    a minimum value of 5 mega bytes.  The default calculation is the same as
    that used by the stone for the STN_FREE_SPACE_THRESHOLD configuration 
    paramenter.
    Minimum: 0
    Maximum: 65536
    Default: 0"

UserGlobals at: #reclaimMinPages put:   40.  "must be >= 1" 
  "Minimum number of pages to process in a single reclaim."

UserGlobals at: #reclaimVerboseLogging put: false.
   "Controls the amount of logging information written to the log file.
    If set to false, normal logging information will be written.  If set to
    true, extra logging information is written to the log file."

UserGlobals at: #sleepTimeBetweenReclaimMs put: 0. "min 0, max 300000"
  "Amount of time (in milliseconds) to sleep after a reclaim that did work."

UserGlobals at: #sleepTimeWithCrBacklogMs put: 0. "min 0, max 300000"
  "Amount of time (in milliseconds) to sleep after a commit when the 
   commit record backlog is larger than 1.25 * the current setting for
   STN_CR_BACKLOG_THRESHOLD.  For each 25 percent above the threshold
   the sleep time is increased so that the reclaimGem does fewer commits 
   the higher the number of commit records is about the threshold."

" AdminGem configuration parameters"

UserGlobals at: #adminVerboseLogging put: false.
   "Controls the amount of logging information written to the log file.
    If set to false, normal logging information will be written.  If set to
    true, extra logging information is written to the log file."

UserGlobals at: #epochGcMaxThreads put: 1.
   "MaxThreads used for next epochGc.
    Minimum: 1.
    Maximum: 32"

UserGlobals at: #epochGcPageBufferSize put: 64.
   "Size in pages of buffer used for epoch GC (must be power of 2).
    Minimum: 8
    Maximum: 1024"

UserGlobals at: #epochGcPercentCpuActiveLimit put: 90.
   "Limit active epoch threads when system percentCpuActive is above this limit.
    Minimum: 0
    Maximum: 100"

UserGlobals at: #epochGcTimeLimit put: 3600.
   "#epochGcTimeLimit controls the maximum frequency of epochs, in seconds.
    For example, the default value set here does not allow more than 
    one epoch every 60 minutes.
    It is recommended that this value not be less than 1800 (i.e. 30 minutes),
    since the aging of objects faulted into gem memory uses 5 minute aging for
    each of 10 subspaces of the pom generation.  Ideally epochGcTimeLimit should
    be several hours."

UserGlobals at: #epochGcTransLimit put: 5000.
   "Minimum number of commits that must have occurred in order to start an
    epoch garbage collection."

UserGlobals at: #saveWriteSetUnionToFile put: false.
   "If true, causes the Admin GC gem to write out the write set union bitmap to
    a binary file before sweeping the union.  The file will be placed in the
    same directory as the Admin GC gem log file."

UserGlobals at: #sweepWsUnionMaxThreads put: 1.
   "MaxThreads used for next wsUnion sweep.
    Minimum: 1
    Maximum: 32"

UserGlobals at: #sweepWsUnionPageBufferSize put: 64.
   "Size in pages of buffer used for wsUnion sweep.
    Minimum: 8
    Maximum: 1024"

UserGlobals at: #sweepWsUnionPercentCpuActiveLimit put: 90.
   "Limit active wsUnion threads when system percentCpuActive is above this limit.
    Minimum: 0
    Maximum: 100"

"NOTE: values which exceed the specified limits (min or max) will be
  cause the garbage collector to ignore the value stored in UserGlobals,
  and the documented limit will be silently used."

'  .
^ true
%

! preserve existing values in renameid parameters
expectvalue true
run
| oldOrDefault |
oldOrDefault  := UserGlobals at: #SweepWsUnionMaxThreads ifAbsent: [1].
UserGlobals at: #sweepWsUnionMaxThreads ifAbsentPut: [oldOrDefault].
oldOrDefault  := UserGlobals at: #SweepWsUnionPageBufferSize ifAbsent: [64].
UserGlobals at: #sweepWsUnionPageBufferSize ifAbsentPut: [oldOrDefault].
oldOrDefault  := UserGlobals at: #SweepWsUnionPercentCpuActiveLimit ifAbsent: [90].
UserGlobals at: #sweepWsUnionPercentCpuActiveLimit ifAbsentPut: [oldOrDefault].
^true
%

! remove all obsolete keys present

expectvalue true
run
UserGlobals removeKey: #jetReclaimMinDeadObjs otherwise: nil .
UserGlobals removeKey: #reclaimMaxPages otherwise: nil .
UserGlobals removeKey: #preheatCacheBeforeCommit otherwise: nil .
UserGlobals removeKey: #GemIOLimit otherwise: nil .
UserGlobals removeKey: #epochGcByteLimit otherwise: nil .
UserGlobals removeKey: #epochGcStatsEnabled otherwise: nil .
UserGlobals removeKey: #deferEpochReclaimThreshold otherwise: nil .
UserGlobals removeKey: #reclaimStatsEnabled otherwise: nil .
UserGlobals removeKey: #reclaimStats otherwise: nil .
UserGlobals removeKey: #epochGcEnabled otherwise: nil .
UserGlobals removeKey: #epochGcStats otherwise: nil .
UserGlobals removeKey: #reclaimDeadCommitLimit otherwise: nil .
UserGlobals removeKey: #promoteDeadLimit otherwise: nil .
UserGlobals removeKey: #deferPromoteDeadReclaimThreshold otherwise: nil .
UserGlobals removeKey: #coordinateJetGemCommits otherwise: nil .
UserGlobals removeKey: #sleepTimeBetweenTranlogWritesMs otherwise: nil .
UserGlobals removeKey: #reclaimDeadShadowPageThreshold otherwise: nil. 
UserGlobals removeKey: #dataPageBufferSize otherwise: nil.
UserGlobals removeKey: #reclaimSleepTime otherwise: nil.  
UserGlobals removeKey: #deferReclaimFreeSpaceThreshold otherwise: nil.
UserGlobals removeKey: #SweepWsUnionMaxThreads otherwise: nil.
UserGlobals removeKey: #SweepWsUnionPageBufferSize otherwise: nil.
UserGlobals removeKey: #SweepWsUnionPercentCpuActiveLimit otherwise: nil.
UserGlobals removeKey: #autoRefreshGcGemConfig otherwise: nil.
UserGlobals removeKey: #enableDebugging otherwise: nil.
UserGlobals removeKey: #verboseLogging otherwise: nil.

^true
%

expectvalue true
run
| oldValueInSeconds |
oldValueInSeconds := UserGlobals at: #sleepTimeBetweenReclaim otherwise: 0 .
oldValueInSeconds _isSmallInteger
  ifFalse:[oldValueInSeconds := 0].
"43521 - don't overwrite the existing value, if any!"
UserGlobals at: #sleepTimeBetweenReclaimMs ifAbsentPut: [oldValueInSeconds * 1000].
UserGlobals removeKey: #sleepTimeBetweenReclaim otherwise: nil .
^true
%

! Now install default values for all new keys, leave old values alone.
!   changed default for epochGcTimeLimit with fix 34388 
expectvalue true
run
UserGlobals at: #reclaimMinPages ifAbsentPut: [40]. 
UserGlobals at: #reclaimMinFreeSpaceMb ifAbsentPut: [0].
UserGlobals at: #sleepTimeBetweenReclaimMs ifAbsentPut: [0].
UserGlobals at: #sleepTimeWithCrBacklogMs ifAbsentPut: [0].
UserGlobals at: #reclaimDeadEnabled  ifAbsentPut: [true].
UserGlobals at: #objsMovedPerCommitThreshold ifAbsentPut: [20000].
UserGlobals at: #deadObjsReclaimedCommitThreshold ifAbsentPut: [20000]. 
UserGlobals at: #maxTransactionDuration ifAbsentPut: [300].
UserGlobals at: #objectTableCleanupIdleTime ifAbsentPut: [0].
UserGlobals at: #deferReclaimCacheDirtyThreshold ifAbsentPut: [75].
UserGlobals at: #epochGcTimeLimit ifAbsentPut: [ 3600 ]. 
UserGlobals at: #epochGcTransLimit ifAbsentPut: [5000]. 
UserGlobals at: #epochGcMaxThreads ifAbsentPut: [1].
UserGlobals at: #epochGcPercentCpuActiveLimit ifAbsentPut: [90].
UserGlobals at: #sweepWsUnionMaxThreads ifAbsentPut: [1].
UserGlobals at: #sweepWsUnionPageBufferSize ifAbsentPut: [64].
UserGlobals at: #sweepWsUnionPercentCpuActiveLimit ifAbsentPut: [90].
UserGlobals at: #saveWriteSetUnionToFile ifAbsentPut: [false]. "Request 43337"
UserGlobals at: #reclaimVerboseLogging ifAbsentPut: [false].
UserGlobals at: #adminVerboseLogging ifAbsentPut: [false].
^ true
%

expectvalue %String
run
" fix 46009 , install/upgrade to a power of 2"
| x v |
x := UserGlobals at: #epochGcPageBufferSize otherwise: nil .
v := 64 . 
x _isSmallInteger ifTrue:[
  [ v < x and:[ v < 512] ] whileTrue:[  
     v := v * 2 .
  ].
].  
UserGlobals at: #epochGcPageBufferSize put: v .
^ 'found ' , x asString , '  installed ' , v asString
%

stack
commit

logout
