! ========================================================================
! Copyright (C) by GemTalk Systems 1991-2020.  All Rights Reserved
!
! $Id: quadbytestring.gs,v 1.14.6.1 2008-04-16 18:43:00 dhenrich Exp $
!
! ========================================================================

! Gs64 v3.0 ,  QuadByteString always created by bom.c

set class QuadByteString
removeallmethods 
removeallclassmethods 

category: 'Documentation'
classmethod:
comment
^ '
A QuadByteString is a string for which each Character occupies four bytes.
Instances are in CPU-native byte order when in memory.

Most of the functionality of this class is inherited from the 
superclass MultiByteString.  See the documentation there for more details'
%

! ------------------- Class methods for QuadByteString

category 'Formatting'
classmethod:
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

category 'Instance Creation'
classmethod:
new: aSize

"Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of two bytes each."

^ self _basicNew:  aSize * 4
%

category 'Instance Creation'
classmethod:
new

"Returns a new instance of the receiver."

^ self _basicNew
%

category 'Instance Creation'
classmethod:
withAll: aString

"Returns an instance of the receiver containing the elements of the argument."
<primitive: 775>
^ super withAll: aString
%

! ------------------- Instance methods for QuadByteString
category: 'Accessing'
method:
size

"Returns the number of Characters in the receiver."

^ self _basicSize // 4 .
%
! fix 41693
category: 'Private'
method:
_basicSize: anInteger

"Disallowed"
self shouldNotImplement: #_basicSize:
%
method:
_basicAt: anInteger put: aValue

"Disallowed"
self shouldNotImplement: #_basicAt:put:
%

category: 'Updating'
method:
size: anInteger

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

^ super _basicSize: 4 * anInteger 
%

category 'Formatting'
method:
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

! deleted collatingTable

category: 'Accessing'
method:
at: anIndex

"Returns the Character at anIndex."

<primitive: 655>
(anIndex _isSmallInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Updating'
method:
at: anIndex put: aChar

"Stores aChar at the specified location and returns aChar."

<primitive: 667>

(aChar class == Character) ifFalse:[
  aChar _validateClass: AbstractCharacter .
  ^ self at: anIndex put: aChar asCharacter
].
(anIndex _isSmallInteger) ifTrue: [
  ((anIndex > (self size + 1)) or: [anIndex <= 0]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
    ].
] ifFalse:[
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #at:put: args: { anIndex . aChar }
%

category: 'Concatenating'
method:
, aCharOrCharColl

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharColl.  The argument
 must be a String, a DoubleByteString, or an AbstractCharacter.

 The result may not be an instance of the class of the receiver if one of the
 following rules applies:

 1) If the receiver is a  QuadByteSymbol, the result  is a  QuadByteString.

 Warning: Creating a new instance and copying the receiver take time.  If you
 can safely modify the receiver, it can be much faster to use the addAll:
 method.  See the documentation of the Concatenating category of class
 SequenceableCollection for more details."

<primitive: 444 >
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^ super , aCharOrCharColl
%

category: 'Adding'
method:
insertAll: aCharOrCharColl at: anIndex

"Inserts aCharOrCharColl at the specified index.  Returns aCharOrCharColl."

<primitive: 447>
anIndex _isSmallInteger ifFalse:[ anIndex _validateClass: Integer ].
((anIndex <= 0) or: [anIndex > (self size + 1)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
aCharOrCharColl _validateClasses: { AbstractCharacter . CharacterCollection }.
^self insertAll: aCharOrCharColl asDoubleByteString at: anIndex.
%

category: 'Converting'
method:
_asDoubleByteString

"Returns a DoubleByteString representation of the receiver if all of the
 Characters in the receiver can be represented as two-byte Characters.
 Returns nil if any of the Characters in the receiver cannot be represented as
 two-byte Characters."

<primitive: 702>
^ self _primitiveFailed: #_asDoubleByteString .
%

category: 'Converting'
method:
asDoubleByteString

 "Returns a DoubleByteString (2 bytes per Character) representation of the receiver
 if all of the Characters in the receiver can be represented as two-byte Characters.
 Otherwise, returns the receiver."

| result |
(result := self _asDoubleByteString) ifNil:[ ^ self ].
^ result
%

category: 'Converting'
method:
asQuadByteString
  ^ self
%

category: 'Adding'
method:
add: aCharOrCharColl

"Appends all of the elements of aCharOrCharColl to the receiver and returns
 aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[ 
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asQuadByteString 
%

category: 'Adding'
method:
addAll: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[ 
  ^ self add: aCharOrCharColl asCharacter .
].
(aCharOrCharColl isKindOf: CharacterCollection) ifTrue:[
  ^ self add: aCharOrCharColl asQuadByteString.
].
aCharOrCharColl do: [:each | self add: each].
^aCharOrCharColl.
%

category: 'Adding'
method:
addLast: aCharOrCharColl

"Equivalent to add: aCharOrCharColl."

<primitive: 776>
aCharOrCharColl class == Character ifTrue:[
  ^ self _primitiveFailed: #add: args: { aCharOrCharColl }
].
(aCharOrCharColl isKindOf: AbstractCharacter) ifTrue:[ 
  ^ self add: aCharOrCharColl asCharacter .
].
aCharOrCharColl _validateClass: CharacterCollection .
^ self add: aCharOrCharColl asQuadByteString 
% 

category: 'Copying'
method:
replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex

"Replaces the elements of the receiver between the indexes startIndex and
 stopIndex inclusive with the elements of aSeqCollection starting at repIndex.
 If aSeqCollection is identical to the receiver, the source and
 destination blocks may overlap.

 Returns the receiver."

<primitive: 777>
| recInfo uStr |
startIndex _isSmallInteger ifFalse:[ startIndex _validateClass: SmallInteger ].
stopIndex _isSmallInteger ifFalse:[ stopIndex _validateClass: SmallInteger ].
repIndex _isSmallInteger ifFalse:[ repIndex _validateClass: SmallInteger ].
recInfo := self _stringCharSize .
(recInfo bitAnd: 16r8) ~~ 0 ifTrue:[  "receivier is a Utf string"
  (uStr := aSeqCollection asUnicodeString) ~~ aSeqCollection ifTrue:[ 
    ^ self replaceFrom: startIndex to: stopIndex with: uStr startingAt: repIndex .
  ].
].
^ super replaceFrom: startIndex to: stopIndex with: aSeqCollection startingAt: repIndex
%

category: 'Deprecated'
method:
asciiLessThan: aString

"Returns true if the receiver collates before the argument using the
 ASCII collating table, which collates AB...Z...ab..z."

self deprecated: 'QuadByteString>>asciiLessThan: deprecated in 3.2.'.
^ self lessThan: aString collatingTable: QuadByteAsciiCollatingTable
%
category: 'Encoding'
method:
encodeAsUTF8

"Encode receiver in UTF8 format.  Returns a Utf8 .
 Generates an error if any character value in receiver exceeds
 the maximum utf32 value 16r10FFFF."

<primitive: 468>
1 to: self size do:[:n | | v |
  v := self wordAt: n .
  v > 16r10FFFF ifTrue:[
    self error:'character at:', n asString , ' exceeds max legal utf 16r10FFFF'
  ].
].
self _primitiveFailed: #encodeAsUTF8
%
category: 'Encoding'
method: 
encodeAsUTF16

"Encode receiver in UTF16 format.  Returns a Utf16 .
 Generates an error if any character value in receiver exceeds
 the maximum utf32 value 16r10FFFF."

<primitive: 1051>
self _primitiveFailed: #encodeAsUTF16
%

method:
encodeAsUTF8IntoString
"Encode receiver in UTF8 format.  Returns a String. For Seaside"

<primitive: 994>
1 to: self size do:[:n | | v |
  v := self wordAt: n .
  v > 16r10FFFF ifTrue:[
    self error:'character at:', n asString , ' exceeds max legal utf 16r10FFFF'
  ].
].
self _primitiveFailed: #encodeAsUTF8IntoString
%

